import os
import podonos
import unittest
from unittest import mock

from podonos.core.file import File
from podonos.common.constant import *


# Mocks HTTP GET request.
def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

    if '/customers/verify/api-key' in args[0]:
        # API key verification
        return MockResponse('true', None, 200)

    if '/version/sdk' in args[0]:
        # SDK versions
        version_response = dict(
            latest="0.1.5",
            recommended="0.1.4",
            minimum="0.1.0")
        return MockResponse(None,
                            version_response,
                            200)

    if '/customers/uploading-presigned-url' in args[0]:
        return MockResponse("https://fake.podonos.com/my_url1", None, 200)

    return MockResponse(None, None, 404)


class TestPodonos(unittest.TestCase):

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_init(self, mock_get):
        # Invalid api_key
        invalid_api_key = '1'
        with self.assertRaises(Exception) as context:
            _ = podonos.init(invalid_api_key)

        # Valid api_key
        valid_api_key = '12345678'
        valid_client = podonos.init(valid_api_key)
        self.assertTrue(isinstance(valid_client, podonos.Client))

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_init_api_key_env(self, mock_get):
        # Valid api_key_env
        os.environ[PODONOS_API_KEY] = "ABCDEFG"
        valid_client = podonos.init()
        self.assertTrue(isinstance(valid_client, podonos.Client))

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_init_both_api_keys(self, mock_get):
        # Valid api_key_env
        os.environ[PODONOS_API_KEY] = "ABCDEFG"
        valid_api_key = '12345678'
        valid_client = podonos.init(valid_api_key)
        self.assertTrue(isinstance(valid_client, podonos.Client))


class TestPodonosClient(unittest.TestCase):

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def setUp(self, mock_get) -> None:
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)

        self.config = {
            'name': 'my_new_model_03272024_p1_k2_en_us',
            'desc': 'Updated model',
            'type': 'NMOS',
            'lan': 'en-us'
        }

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_create_evaluator(self, mock_get):
        # Missing name is allowed.
        etor = self._mock_client.create_evaluator(
            desc=self.config['desc'],
            type=self.config['type'],
            lan=self.config['lan'])
        self.assertIsNotNone(etor)

        # Too short name.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(
                name='a',
                desc=self.config['desc'],
                type=self.config['type'],
                lan=self.config['lan'])

        # Missing description is allowed.
        etor = self._mock_client.create_evaluator(
            name=self.config['name'],
            type=self.config['type'],
            lan=self.config['lan'])
        self.assertIsNotNone(etor)

        # Unknown type.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(
                name=self.config['name'],
                desc=self.config['desc'],
                type="unknown_type",
                lan=self.config['lan'])

        # Missing language is allowed.
        etor = self._mock_client.create_evaluator(
            name=self.config['name'],
            desc=self.config['desc'],
            type=self.config['type'])
        self.assertIsNotNone(etor)

        # Invalid language.
        with self.assertRaises(ValueError) as context:
            _ = self._mock_client.create_evaluator(
                name=self.config['name'],
                desc=self.config['desc'],
                type=self.config['type'],
                lan='abcd')

        # Valid configuration
        etor = self._mock_client.create_evaluator(
            name=self.config['name'],
            desc=self.config['desc'],
            type=self.config['type'],
            lan=self.config['lan'])
        self.assertIsNotNone(etor)

        # P.808
        etor = self._mock_client.create_evaluator(
            name=self.config['name'],
            desc=self.config['desc'],
            type='P808',
            lan=self.config['lan'])
        self.assertIsNotNone(etor)

        # SMOS
        etor = self._mock_client.create_evaluator(
            name=self.config['name'],
            desc=self.config['desc'],
            type='SMOS',
            lan=self.config['lan'])
        self.assertIsNotNone(etor)


class TestPodonosEvaluator(unittest.TestCase):

    @unittest.skip
    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_evaluator(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)

        self.config = {
            'name': 'my_new_model_03272024_p1_k2_en_us',
            'desc': 'Updated model',
            'type': 'NMOS',
            'lan': 'en-us'
        }

        etor = self._mock_client.create_evaluator(
            name='new_model', desc='', type='NMOS', lan='en-au')
        self.assertIsNotNone(etor)

        some_file = os.path.join(os.path.dirname(__file__), '../speech_0_0.mp3')
        etor.add_file(File(path=some_file, tags=['unknown_file,new_model']))
        etor.close()


if __name__ == '__main__':
    unittest.main(verbosity=2)



from dataclasses import dataclass
from typing import List


@dataclass
class StimulusStatsFile:
    name: str
    tags: List[str]
    
    @staticmethod
    def from_dict(data: dict) -> 'StimulusStatsFile':
        required_keys = ["name", "tags"]
        for key in required_keys:
            if key not in data:
                raise ValueError(f"Invalid data format for StimulusStatsFile: {data}")
        return StimulusStatsFile(
            name=data["name"],
            tags=data["tags"]
        )
        
    def to_dict(self) -> dict:
        return {
            "name": self.name,
            "tags": self.tags
        }
    
@dataclass
class StimulusStats:
    files: List[StimulusStatsFile]
    mean: float
    median: float
    std: float
    ci_90: float
    ci_95: float
    ci_99: float
    
    @staticmethod
    def from_dict(data: dict) -> 'StimulusStats':
        required_keys = ["files", "mean", "median", "std", "ci_90", "ci_95", "ci_99"]
        for key in required_keys:
            if key not in data:
                raise ValueError(f"Invalid data format for StimulusStats: {data}")
        
        files = [StimulusStatsFile.from_dict(file) for file in data["files"]]
        return StimulusStats(
            files=files,
            mean=data['mean'],
            median=data['median'],
            std=data['std'],
            ci_90=data['ci_90'],
            ci_95=data['ci_95'],
            ci_99=data['ci_99']
        )
    
    def to_dict(self) -> dict:
        return {
            "files": [file.to_dict() for file in self.files],
            "mean": self.mean,
            "median": self.median,
            "std": self.std,
            "ci_90": self.ci_90,
            "ci_95": self.ci_95,
            "ci_99": self.ci_99
        }


import logging
from typing import Union, List

from podonos.common.enum import EvalType, QuestionFileType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from podonos.errors.error import NotSupportedError

# For logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

class SingleStimulusEvaluator(Evaluator):
    def __init__(self, supported_evaluation_type: List[EvalType], api_client: APIClient, eval_config: Union[EvalConfig, None] = None):
        super().__init__(api_client, eval_config)
        self._supported_evaluation_type = supported_evaluation_type
    
    def add_file(
        self, 
        file: File
    ) -> None:
        """Add new file for speech evaluation.
        The file may be either in {wav, mp3} format. The file will be securely uploaded to
        Podonos service system.

        Args:
            file: Object including path and tags

        Example:
        If you want to evaluate each audio file separately (e.g., Naturalness MOS):
            add_file(file=File(path='./test.wav', tags=['male', 'generated']))

        Returns: None

        Raises:
            ValueError: if this function is called before calling init().
            FileNotFoundError: if a given file is not found.
        """

        if not self._initialized:
            raise ValueError("Try to add file once the evaluator is closed.")

        eval_config = self._get_eval_config()
        if eval_config.eval_type in self._supported_evaluation_type:
            audio = self._set_audio(path=file.path, tags=file.tags, group=None, type=QuestionFileType.STIMULUS)
            self._eval_audios.append([audio])
    
    def add_file_pair(self, target: File, ref: File) -> None:
        raise NotSupportedError("The 'add_file_pair' is only supported in these evaluation types: {'CMOS', 'DMOS'}")
    
    def add_file_set(self, file0: File, file1: File) -> None:
        raise NotSupportedError("The 'add_file_set' is only supported in these evaluation types: {'SMOS', 'PREF'}")


from typing import Any, Dict, Optional, List

from requests import HTTPError

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig, EvalConfigDefault
from podonos.core.evaluation import Evaluation
from podonos.core.evaluator import Evaluator
from podonos.core.stimulus_stats import StimulusStats
from podonos.evaluators.double_stimuli_evaluator import DoubleStimuliEvaluator
from podonos.evaluators.single_stimulus_evaluator import SingleStimulusEvaluator

class Client:
    """Podonos Client class. Used for creating individual evaluator and managing the evaluations."""

    _api_client: APIClient
    _initialized: bool = False
    
    def __init__(self, api_client: APIClient):
        self._api_client = api_client
        self._initialized = True

    def create_evaluator(
        self,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        type: str = EvalConfigDefault.TYPE.value,
        lan: str = EvalConfigDefault.LAN.value,
        granularity: float = EvalConfigDefault.GRANULARITY,
        num_eval: int = EvalConfigDefault.NUM_EVAL,
        due_hours: int = EvalConfigDefault.DUE_HOURS,
        auto_start: bool = EvalConfigDefault.AUTO_START
    ) -> Evaluator:
        """Creates a new evaluator with a unique evaluation session ID.
        For the language code, see https://docs.dyspatch.io/localization/supported_languages/

        Args:
            name: This session name. Its length must be > 1. If empty, a random name is used. Optional.
            desc: Description of this session. Optional.
            type: Evaluation type. One of {'NMOS', 'SMOS', 'P808'}. Default: NMOS
            lan: Human language for this audio. One of those in Language. Default: en-us
            num_eval: The minimum number of repetition for each audio evaluation. Should be >=1. Default: 3.
            due_hours: An expected number of days of finishing this mission and getting the evaluation report.
                        Must be >= 12. Default: 12.
            auto_start: The evaluation start automatically if True. Manually start in the workspace otherwise.

        Returns:
            Evaluator instance.

        Raises:
            ValueError: if this function is called before calling init().
        """

        if not self._initialized:
            raise ValueError("This function is called before initialization.")
        
        if not EvalType.is_eval_type(type):
            raise ValueError("Not supported evaluation types. Use one of the {'NMOS', 'QMOS', 'P808', 'SMOS'}")
        
        eval_config = EvalConfig(
            name=name, desc=desc,
            type=type, lan=lan,
            granularity=granularity, num_eval=num_eval,
            due_hours=due_hours, auto_start=auto_start
        )
        if type in [EvalType.SMOS.value, EvalType.PREF.value]:
            return DoubleStimuliEvaluator(
                supported_evaluation_type=[EvalType.SMOS, EvalType.PREF],
                api_client=self._api_client,
                eval_config=eval_config
            )
        
        return SingleStimulusEvaluator(
            supported_evaluation_type=[EvalType.NMOS, EvalType.QMOS, EvalType.P808],
            api_client=self._api_client,
            eval_config=eval_config
        )
    
    def get_evaluation_list(self) -> List[Dict[str, Any]]:
        """Gets a list of evaluations.

        Args: None

        Returns:
            Evaluation containing all the evaluation info
        """
        try:
            response = self._api_client.get("evaluations")
            response.raise_for_status()
            evaluations = [Evaluation.from_dict(evaluation) for evaluation in response.json()]
            return [evaluation.to_dict() for evaluation in evaluations]
        except Exception as e:
            raise HTTPError(f"Failed to get evaluation list: {e}")
    
    def get_stats_dict_by_id(self, evaluation_id: str) -> List[Dict[str, Any]]:
        """Gets a list of evaluation statistics referenced by id.

        Args:
            evaluation_id: Evaluation id. See get_evaluation_list() above.

        Returns:
            List of statistics for the evaluation.
        """
        try:
            response = self._api_client.get(f"evaluations/{evaluation_id}/stats")
            response.raise_for_status()
            stats = [StimulusStats.from_dict(stats) for stats in response.json()]
            return [stat.to_dict() for stat in stats]
        except Exception as e:
            raise HTTPError(f"Failed to get stimulus stats: {e}")
    
    def download_stats_csv_by_id(self, evaluation_id: str, output_path: str) -> None:
        """Downloads the evaluation statistics into CSV referenced by id.

        Args:
            evaluation_id: Evaluation id. See get_evaluation_list() above.
            output_path: Path to the output CSV.

        Returns: None
        """
        stats = self.get_stats_dict_by_id(evaluation_id)
        with open(output_path, "w") as f:
            f.write("name,tags,mean,median,std,ci_90,ci_95,ci_99\n")
            for stat in stats:
                for file in stat['files']:
                    tags = ";".join(file["tags"])
                    f.write(f"{file['name']},{tags},{stat['mean']},{stat['median']},{stat['std']},{stat['ci_90']},{stat['ci_95']},{stat['ci_99']}\n")



"""Podonos SDK for speech/audio evaluation.
For details, please refer to https://github.com/podonos/pysdk/
"""

import time
import logging
import os

from podonos.common.constant import *
from podonos.core.api import APIClient
from podonos.core.client import Client

# For logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


class Podonos:
    """Class for Podonos SDK."""
    _api_key: str
    _api_base_url: str
    _api_client: APIClient
    _initialized: bool = False

    @staticmethod
    def init(
        api_key: str = None,
        api_url: str = PODONOS_API_BASE_URL
    ) -> Client:
        """ Initializes the SDK. This function must be called before calling other functions.
            Raises error on invalid or missing API key. Also, raises exception on other failures.
            api_key: API Key. If not set, try to read PODONOS_API_KEY. If both are not set, raises an error. Optional.
            api_url: URL for API access. Optional.

            Returns: None

            Raises: ValueError: if neither the API Key nor PODONOS_API_KEY is not set or invalid.
        """

        # If this is initialized multiple times, it sounds suspicious.
        if Podonos._initialized:
            log.debug('You are initializing >1 times.')

        api_key_env = None
        if os.environ.get(PODONOS_API_KEY):
            api_key_env = os.environ[PODONOS_API_KEY]

        if api_key is None and api_key_env is None:
            raise ValueError(TerminalColor.FAIL +
                             f"Podonos API key is missing. Please set api_key or "
                             f"{PODONOS_API_KEY} environment variable.\n"
                             f"For details, please visit {PODONOS_DOCS_API_KEY}." + TerminalColor.ENDC)

        if api_key is not None and api_key_env is not None:
            print(TerminalColor.WARN +
                  f"Both api_key and {PODONOS_API_KEY} environment variable are set. "
                  f"Uses api_key." + TerminalColor.FAIL)

        if api_key is None:
            api_key = api_key_env

        # API key verification.
        if len(api_key) <= 3:
            raise ValueError(TerminalColor.FAIL +
                             f"The API key {api_key} is not valid. \n"
                             f"Please use a valid API key or visit {PODONOS_HOME}." + TerminalColor.ENDC
            )

        api_client = APIClient(api_key, api_url)
        
        Podonos._api_client = api_client
        Podonos._initialized = api_client.initialize()
        return Client(api_client)



import os
import unittest
import requests
from unittest.mock import Mock, patch, MagicMock
from typing import Optional
from datetime import datetime

from podonos.common.enum import QuestionFileType
from podonos.common.exception import HTTPError
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from tests.test_audio import TESTDATA_SPEECH_CH1_MP3

class MockEvaluator(Evaluator):
    def __init__(self, api_client = Mock(spec=APIClient), eval_config: Optional[EvalConfig] = None):
        super().__init__(api_client, eval_config)

    def add_file(self, file: File) -> None:
        pass
    
    def add_file_pair(self, target: File, ref: File) -> None:
        pass
    
    def add_file_set(self, file0: File, file1: File) -> None:
        pass
    
class TestEvaluator(unittest.TestCase):
    
    def setUp(self):
        self.eval_config = EvalConfig(type='NMOS')
        self.evaluator = MockEvaluator(eval_config=self.eval_config)

    def test_get_eval_config_success(self):
        eval_config = EvalConfig(type='NMOS')
        evaluator = MockEvaluator(eval_config=eval_config)
        
        result = evaluator._get_eval_config()
        self.assertEqual(result, eval_config)

    def test_get_eval_config_not_initialized(self):
        evaluator = MockEvaluator(eval_config=None)
        
        with self.assertRaises(ValueError) as context:
            evaluator._get_eval_config()
        
        self.assertEqual(str(context.exception), "Evaluator is not initialized")

    def test_create_evaluation_success(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = None  # No exception
        mock_response.json.return_value = {
            "id": "123",
            "title": "title",
            "internal_name": None,
            "description": None,
            "status": "DRAFT",
            "created_time": datetime.now().isoformat(),
            "updated_time": datetime.now().isoformat(),
        }
        
        eval_config = EvalConfig("NMOS")
        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=eval_config)
        evaluator._api_client.post.return_value = mock_response # type: ignore

        result = evaluator._create_evaluation()

        self.assertEqual(result.id, "123")
        self.assertEqual(eval_config.eval_id, "123")

    def test_create_evaluation_http_error(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(response=MagicMock(status_code=400))

        eval_config = EvalConfig("NMOS")
        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=eval_config)
        evaluator._api_client.post.return_value = mock_response # type: ignore

        with self.assertRaises(HTTPError) as context:
            evaluator._create_evaluation()

        self.assertIn("Failed to create", str(context.exception))

    def test_post_request_evaluation_success(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = MagicMock(status_code=200)
        
        eval_config = EvalConfig("NMOS")
        evaluator = MockEvaluator(eval_config=eval_config)
        evaluator._api_client.post.return_value = mock_response # type: ignore
        
        evaluator._post_request_evaluation()
        evaluator._api_client.post.assert_called_once_with("request-evaluation", { # type: ignore
            'eval_id': eval_config.eval_id,
            'eval_type': 'SPEECH_NMOS'
        })

    def test_post_request_evaluation_http_error(self):
        mock_response = MagicMock()
        mock_response.raise_for_status.side_effect = requests.exceptions.HTTPError(response=MagicMock(status_code=400))

        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=EvalConfig("NMOS"))
        evaluator._api_client.post.return_value = mock_response # type: ignore

        with self.assertRaises(HTTPError) as context:
            evaluator._post_request_evaluation()
        
        self.assertIn("Failed to request", str(context.exception))

    @patch('os.path.isfile', return_value=True)
    @patch('os.access', return_value=True)
    def test_set_audio(self, mock_access, mock_isfile):
        path = TESTDATA_SPEECH_CH1_MP3
        tags = ["tag1", "tag2"]
        group = "group1"
        type = QuestionFileType.STIMULUS
        
        audio = self.evaluator._set_audio(path, tags, group, type)
        
        expected_name = "speech_ch1.mp3"
        expected_remote_name = os.path.join(self.eval_config.eval_creation_timestamp, expected_name)
        
        self.assertEqual(audio.path, path)
        self.assertEqual(audio.name, expected_name)
        self.assertEqual(audio.remote_name, expected_remote_name)
        self.assertEqual(audio.tags, tags)
        self.assertEqual(audio.group, group)
        self.assertEqual(audio.type, type)

    @patch('os.path.isfile')
    @patch('os.access')
    def test_validate_path_success(self, mock_access, mock_isfile):
        mock_isfile.return_value = True
        mock_access.return_value = True

        evaluator = MockEvaluator(eval_config=EvalConfig(type='NMOS'))
        
        valid_path = '/valid/path/to/file.txt'
        result = evaluator._validate_path(valid_path)
        self.assertEqual(result, valid_path)
        
        mock_isfile.assert_called_once_with(valid_path)
        mock_access.assert_called_once_with(valid_path, os.R_OK)

    @patch('os.path.isfile')
    @patch('os.access')
    def test_validate_path_file_not_exist(self, mock_access, mock_isfile):
        mock_isfile.return_value = False
        evaluator = MockEvaluator(api_client=MagicMock(), eval_config=EvalConfig(type='NMOS'))
        
        invalid_path = '/invalid/path/to/file.txt'
        with self.assertRaises(FileNotFoundError) as context:
            evaluator._validate_path(invalid_path)
        
        self.assertIn("doesn't exist", str(context.exception))
        
        mock_isfile.assert_called_once_with(invalid_path)
        mock_access.assert_not_called()

    @patch('os.path.isfile')
    @patch('os.access')
    def test_validate_path_not_readable(self, mock_access, mock_isfile):
        mock_isfile.return_value = True
        mock_access.return_value = False

        evaluator = MockEvaluator(eval_config=EvalConfig(type='NMOS'))
        
        unreadable_path = '/unreadable/path/to/file.txt'
        with self.assertRaises(FileNotFoundError) as context:
            evaluator._validate_path(unreadable_path)
        
        self.assertIn("isn't readable", str(context.exception))
        
        mock_isfile.assert_called_once_with(unreadable_path)
        mock_access.assert_called_once_with(unreadable_path, os.R_OK)
    
    def test_get_name_and_remote_name(self):
        valid_path = "/path/to/file.txt"
        expected_name = "file.txt"
        expected_remote_name = os.path.join(self.eval_config.eval_creation_timestamp, expected_name)
        
        name, remote_name = self.evaluator._get_name_and_remote_name(valid_path)
        
        self.assertEqual(name, expected_name)
        self.assertEqual(remote_name, expected_remote_name)

if __name__ == '__main__':
    unittest.main()



import pytest
from unittest.mock import Mock

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.file import File
from podonos.errors.error import NotSupportedError
from podonos.evaluators.double_stimuli_evaluator import DoubleStimuliEvaluator


class TestDoubleStimuliEvaluator:
    def setup_method(self):
        self.api_client = Mock(spec=APIClient)
        self.eval_config = EvalConfig(type='SMOS')
        self.evaluator = DoubleStimuliEvaluator(
            supported_evaluation_type=[EvalType.SMOS],
            api_client=self.api_client,
            eval_config=self.eval_config
        )

    def test_add_file_pair_without_init(self):
        self.evaluator._initialized = False
        target_audio_config = File(path='generated.wav', tags=['target'])
        ref_audio_config = File(path='original.wav', tags=['reference'])

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_file_pair(target=target_audio_config, ref=ref_audio_config)
        
        assert 'Try to add_file_pair once the evaluator is closed.' in str(excinfo.value)
    
    def test_add_file_not_supported(self):
        with pytest.raises(NotSupportedError) as excinfo:
            self.evaluator.add_file(File(path="1.wav", tags=["test"]))
        assert "The 'add_file' is only supported in these evaluation types: {'NMOS', 'QMOS', 'P808'}" in str(excinfo.value)

    def test_add_file_pair_unsupported_eval_type(self):
        self.evaluator._initialized = True

        target = File(path='target.wav', tags=['generated'])
        ref = File(path='ref.wav', tags=['human'])

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_file_pair(target=target, ref=ref)

        assert "The add_file_pair function is used for 'CMOS', 'DMOS'" in str(excinfo.value)

    def test_add_file_set_unsupported_eval_type(self):
        self.evaluator._initialized = True
        self.eval_config._eval_type = EvalType.CMOS

        file0 = File(path='file1.wav', tags=['file1'])
        file1 = File(path='file2.wav', tags=['file2'])

        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_file_set(file0=file0, file1=file1)

        assert "The add_file_set function is used for 'SMOS', 'PREF'" in str(excinfo.value)

    def test_close_upsupported_eval_type(self):
        self.evaluator._initialized = True
        self.eval_config._eval_type = EvalType.PREF

        file0 = File(path='file1.wav', tags=['file1'])
        file1 = File(path='file2.wav', tags=['file2'])
        self.evaluator.add_file_set(file0=file0, file1=file1)
        
        with pytest.raises(ValueError) as excinfo:
            self.evaluator.close()

        assert "Not supported evaluation type" in str(excinfo.value)

    def test_generate_random_group_name(self):
        group_name = self.evaluator._generate_random_group_name()
        parts = group_name.split('_')
        assert len(parts) == 2
        assert parts[0].isdigit()
        assert len(parts[1]) == 36 



import logging
import time
import uuid
from typing import Union, List

from podonos.common.enum import EvalType, QuestionFileType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.evaluator import Evaluator
from podonos.core.file import File
from podonos.errors.error import NotSupportedError

# For logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

class DoubleStimuliEvaluator(Evaluator):
    def __init__(self, supported_evaluation_type: List[EvalType], api_client: APIClient, eval_config: Union[EvalConfig, None] = None):
        super().__init__(api_client, eval_config)
        self._supported_evaluation_type = supported_evaluation_type
    
    def add_file(self, file: File) -> None:
        raise NotSupportedError("The 'add_file' is only supported in these evaluation types: {'NMOS', 'QMOS', 'P808'}")
    
    def add_file_pair(self, target: File, ref: File) -> None:
        """Adds new files for speech evaluation of CMOS and DMOS
        The files may be either in {wav, mp3} format. The files will be securely uploaded to
        Podonos service system.

        Args:
            target: The target audio file configuration. This is the audio file
                    that is being evaluated to determine if it is better or worse
                    than the reference file.
            ref:    The reference audio file configuration. This is the audio file
                    that serves as the standard or baseline for comparison with the
                    target audio file.

        Example:
        If you want to evaluate audio files together (e.g., Comparative MOS):
            target_file, ref_file = File(path="generated.wav", tags=['target']), File(path="original.wav", tags=['reference'])
            add_file_pair(target=target_file, ref=ref_file)

        Returns: None

        Raises:
            ValueError: if this function is called before calling init().
            FileNotFoundError: if a given file is not found.
        """
        
        if not self._initialized:
            raise ValueError("Try to add_file_pair once the evaluator is closed.")
        
        eval_config = self._get_eval_config()
        if eval_config.eval_type not in [EvalType.CMOS, EvalType.DMOS]:
            raise ValueError("The add_file_pair function is used for 'CMOS', 'DMOS'")
        
        if eval_config.eval_type in self._supported_evaluation_type:
            group = self._generate_random_group_name()
            target_audio = self._set_audio(path=target.path, tags=target.tags, group=group, type=QuestionFileType.STIMULUS)
            ref_audio = self._set_audio(path=ref.path, tags=ref.tags, group=group, type=QuestionFileType.REF)
            self._eval_audios.append([target_audio, ref_audio])
    
    def add_file_set(self, file0: File, file1: File) -> None:
        """Adds a new set of files for evaluation.
        This function adds a set of files for evaluation purposes. The files may be either in {wav, mp3} format.
        The files will be securely uploaded to the Podonos service system.

        Args:
            file1: The first audio file configuration. This can be either a target or a reference file
                depending on the evaluation configuration.
            file2: The second audio file configuration. This can be either a target or a reference file
                depending on the evaluation configuration.

        Example:
        If you want to evaluate audio files together (e.g., Similarity MOS):
            file1, file2 = File(path="file1.wav", tags=['file1']), File(path="file2.wav", tags=['file2'])
            add_file_set(file1=file1, file2=file2)

        Returns: None

        Raises:
            ValueError: if this function is called before calling init().
            FileNotFoundError: if a given file is not found.
        """
        
        if not self._initialized:
            raise ValueError("Try to add_file_set once the evaluator is closed.")
        
        eval_config = self._get_eval_config()
        if eval_config.eval_type not in [EvalType.SMOS, EvalType.PREF]:
            raise ValueError("The add_file_set function is used for 'SMOS', 'PREF'")
        
        if eval_config.eval_type in self._supported_evaluation_type:
            group = self._generate_random_group_name()
            audio1 = self._set_audio(path=file0.path, tags=file0.tags, group=group, type=QuestionFileType.STIMULUS)
            audio2 = self._set_audio(path=file1.path, tags=file1.tags, group=group, type=QuestionFileType.STIMULUS)
            self._eval_audios.append([audio1, audio2])

    def _generate_random_group_name(self) -> str:
        current_time_milliseconds = int(time.time() * 1000)
        random_uuid = uuid.uuid4()
        return f"{current_time_milliseconds}_{random_uuid}"


import pytest
from unittest.mock import Mock

from podonos.common.enum import EvalType
from podonos.core.api import APIClient
from podonos.core.config import EvalConfig
from podonos.core.file import File
from podonos.errors.error import NotSupportedError
from podonos.evaluators.single_stimulus_evaluator import SingleStimulusEvaluator

class TestSingleStimulusEvaluator:
    def setup_method(self):
        self.api_client = Mock(spec=APIClient)
        self.eval_config = EvalConfig(type='NMOS')
        self.evaluator = SingleStimulusEvaluator(
            supported_evaluation_type=[EvalType.NMOS, EvalType.QMOS, EvalType.P808],
            api_client=self.api_client,
            eval_config=self.eval_config
        )

    def test_add_file_before_initialized(self):
        self.evaluator._initialized = False
        with pytest.raises(ValueError) as excinfo:
            self.evaluator.add_file(file=File(path='./male.wav', tags=['male']))
        assert 'Try to add file once the evaluator is closed.' in str(excinfo.value)

    def test_add_file_pair_not_supported(self):
        target_audio_config = File(path='target.wav', tags=['target'])
        ref_audio_config = File(path='ref.wav', tags=['ref'])
        with pytest.raises(NotSupportedError) as excinfo:
            self.evaluator.add_file_pair(target=target_audio_config, ref=ref_audio_config)
        assert "The 'add_file_pair' is only supported in these evaluation types: {'CMOS', 'DMOS'}" in str(excinfo.value)

    def test_add_file_set_not_supported(self):
        file0 = File(path='original.wav', tags=['original'])
        file1 = File(path='generated.wav', tags=['generated'])
        with pytest.raises(NotSupportedError) as excinfo:
            self.evaluator.add_file_set(file0=file0, file1=file1)
        assert "The 'add_file_set' is only supported in these evaluation types: {'SMOS', 'PREF'}" in str(excinfo.value)



"""
Default constant values across whole SDK
"""


# Text colors on terminal
class TerminalColor:
    BOLD = '\033[1m'
    ENDC = '\033[0m'
    FAIL = '\033[91m'
    OK = '\033[92m'
    WARN = '\033[93m'

# Podonos homepage
PODONOS_HOME = 'https://www.podonos.com/'

# Podonos docs API key
PODONOS_DOCS_API_KEY = 'https://docs.podonos.com/apikey'

# API key environment variable
PODONOS_API_KEY = 'PODONOS_API_KEY'

# Podonos Workspace
PODONOS_WORKSPACE = "https://workspace.podonos.com"

# Podonos API base URL
PODONOS_API_BASE_URL = "https://prod.podonosapi.com"

# Contact email
PODONOS_CONTACT_EMAIL = "hello@podonos.com"



import os
import unittest
from podonos.core.audio import AudioMeta

TESTDATA_SPEECH_CH1_FLAC = os.path.join(os.path.dirname(__file__), 'speech_ch1.flac')
TESTDATA_SPEECH_CH1_MP3 = os.path.join(os.path.dirname(__file__), 'speech_ch1.mp3')
TESTDATA_SPEECH_TWO_CH1_WAV = os.path.join(os.path.dirname(__file__), 'speech_two_ch1.wav')
TESTDATA_SPEECH_TWO_CH2_WAV = os.path.join(os.path.dirname(__file__), 'speech_two_ch2.wav')


class TestAudioMeta(unittest.TestCase):
    def test_audio_meta_ch1_mp3(self):
        meta = AudioMeta(TESTDATA_SPEECH_CH1_MP3)
        self.assertTrue(meta.nchannels == 1)
        self.assertTrue(meta.framerate == 24000)
        self.assertTrue(meta.duration_in_ms == 2935)
        
    def test_audio_meta_ch1_wav(self):
        meta = AudioMeta(TESTDATA_SPEECH_TWO_CH1_WAV)
        self.assertTrue(meta.nchannels == 1)
        self.assertTrue(meta.framerate == 16000)
        self.assertTrue(meta.duration_in_ms == 558)

    def test_audio_meta_ch2_wav(self):
        meta = AudioMeta(TESTDATA_SPEECH_TWO_CH2_WAV)
        self.assertTrue(meta.nchannels == 2)
        self.assertTrue(meta.framerate == 16000)
        self.assertTrue(meta.duration_in_ms == 558)
        
    def test_audio_meta_unsupported_format(self):
        with self.assertRaises(AssertionError) as context:
            AudioMeta(TESTDATA_SPEECH_CH1_FLAC)
        self.assertTrue('Unsupported file format' in str(context.exception))

if __name__ == '__main__':
    unittest.main(verbosity=2)



import os
import podonos
import unittest
from unittest import mock


# Mocks HTTP GET request.
def mocked_requests_get(*args, **kwargs):
    class MockResponse:
        def __init__(self, response_text, response_json, status_code):
            self.text = response_text
            self.json_response = response_json
            self.status_code = status_code

        def json(self):
            return self.json_response

        def raise_for_status(self):
            return

    if '/customers/verify/api-key' in args[0]:
        # API key verification
        return MockResponse('true', None, 200)

    if '/version/sdk' in args[0]:
        # SDK versions
        version_response = dict(
            latest="0.1.5",
            recommended="0.1.4",
            minimum="0.1.0")
        return MockResponse(None,
                            version_response,
                            200)

    if '/evaluations' in args[0] and '/stats' in args[0]:
        # Stats by id
        evaluation_stats = [dict(
            files=[{"name": "tr16.wav", "tags": ["generated"]}], mean=3.4, median=3.5, std=1.07,
            ci_90=1.14, ci_95=1.48, ci_99=1.53
        )]
        return MockResponse(None, evaluation_stats, 200)

    if '/evaluations' in args[0]:
        # Evaluation list
        evaluation_list = [dict(
            id="mock_id",
            title="mock_title",
            internal_name="mock_internal_name",
            description="mock_desc",
            status="mock_status",
            created_time="2024-05-21T06:18:09.659270Z",
            updated_time="2024-05-21T06:18:09.659270Z",
        )]
        return MockResponse(None, evaluation_list, 200)

    return MockResponse(None, None, 404)


class TestEvaluationClient(unittest.TestCase):

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_evaluation_list(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        response = self._mock_client.get_evaluation_list()
        self.assertTrue(isinstance(response, list))
        self.assertTrue(len(response) > 0)
        json = response[0]
        self.assertTrue('id' in json)
        self.assertTrue('title' in json)
        self.assertTrue('internal_name' in json)
        self.assertTrue('description' in json)
        self.assertTrue('status' in json)
        self.assertTrue('created_time' in json)
        self.assertTrue('updated_time' in json)

    @mock.patch('requests.get', side_effect=mocked_requests_get)
    def test_stimulus_stats_by_id(self, mock_get):
        valid_api_key = "1234567890"
        self._mock_client = podonos.init(api_key=valid_api_key)
        response = self._mock_client.get_stats_dict_by_id(evaluation_id="mock_id")
        self.assertTrue(isinstance(response, list))
        self.assertTrue(len(response) > 0)
        json = response[0]
        self.assertTrue('files' in json)
        self.assertTrue('mean' in json)
        self.assertTrue('median' in json)
        self.assertTrue('std' in json)
        self.assertTrue('ci_90' in json)
        self.assertTrue('ci_95' in json)
        self.assertTrue('ci_99' in json)


if __name__ == '__main__':
    unittest.main(verbosity=2)



import unittest
from unittest.mock import patch, MagicMock
from podonos.core.api import APIClient

class TestAPIClient(unittest.TestCase):
    def setUp(self):
        self.api_key = 'test_api_key'
        self.api_url = 'http://testapi.com'
        self.client = APIClient(self.api_key, self.api_url)

    @patch('requests.get')
    def test_get_success(self, mock_get):
        mock_response = MagicMock()
        mock_response.text = "true"
        mock_get.return_value = mock_response

        response = self.client.get('test-endpoint')
        self.assertEqual(response.text, "true")
        mock_get.assert_called_once_with(f'{self.api_url}/test-endpoint', headers=self.client._headers, params=None)

    @patch('requests.post')
    def test_post_success(self, mock_post):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_post.return_value = mock_response

        data = {'key': 'value'}
        response = self.client.post('test-endpoint', data)
        self.assertEqual(response.status_code, 200)
        mock_post.assert_called_once_with(f'{self.api_url}/test-endpoint', headers=self.client._headers, json=data)

    @patch('requests.put')
    def test_put_success(self, mock_put):
        mock_response = MagicMock()
        mock_response.status_code = 200
        mock_put.return_value = mock_response

        data = {'key': 'value'}
        response = self.client.put('test-endpoint', data)
        self.assertEqual(response.status_code, 200)
        mock_put.assert_called_once_with(f'{self.api_url}/test-endpoint', headers=self.client._headers, json=data)

    @patch('podonos.core.api.APIClient._check_minimum_version')
    @patch('podonos.core.api.APIClient.get')
    def test_initialize_success(self, mock_get, mock_check_minimum_version):
        mock_response = MagicMock()
        mock_response.text = "true"
        mock_get.return_value = mock_response
        mock_check_minimum_version.return_value = True

        result = self.client.initialize()
        self.assertTrue(result)
        mock_get.assert_called_once_with(f'customers/verify/api-key')
    
    @patch('podonos.core.api.APIClient._check_minimum_version')
    @patch('podonos.core.api.APIClient.get')
    def test_initialize_invalid_api_key(self, mock_get, mock_check_minimum_version):
        mock_response = MagicMock()
        mock_response.text = "false"
        mock_get.return_value = mock_response
        mock_check_minimum_version.return_value = True

        with self.assertRaises(ValueError) as context:
            self.client.initialize()

        self.assertIn("Invalid API key", str(context.exception))
        mock_get.assert_called_once_with('customers/verify/api-key')

    def test_add_headers(self):
        self.client.add_headers('New-Header', 'HeaderValue')
        self.assertIn('New-Header', self.client._headers)
        self.assertEqual(self.client._headers['New-Header'], 'HeaderValue')

    @patch('importlib.metadata.version')
    def test_get_podonos_version_success(self, mock_version):
        mock_version.return_value = "1.2.3"
        
        result = self.client._get_podonos_version()
        self.assertEqual(result, "1.2.3")
        mock_version.assert_called_once_with("podonos")

if __name__ == '__main__':
    unittest.main()



import soundfile as sf
from pathlib import Path
from typing import Tuple, Optional, Dict, Any, List

from podonos.common.enum import QuestionFileType
from podonos.errors.error import InvalidFileError

class AudioMeta:
    _nchannels: int
    _framerate: int
    _duration_in_ms: int

    def __init__(self, path: str) -> None:
        self._nchannels, self._framerate, self._duration_in_ms = self._set_audio_meta(path)

    @property
    def nchannels(self) -> int:
        return self._nchannels

    @property
    def framerate(self) -> int:
        return self._framerate

    @property
    def duration_in_ms(self) -> int:
        return self._duration_in_ms

    def _set_audio_meta(self, path: str) -> Tuple[int, int, int]:
        """ Gets info from an audio file.

        Returns:
            nchannels: Number of channels
            framerate: Number of frames per second. Same as the sampling rate.
            duration_in_ms: Total length of the audio in milliseconds

        Raises:
            FileNotFoundError: if the file is not found.
            wave.Error: if the file doesn't read properly.
            AssertionError: if the file format is not wav.
        """

        # Check if this is wav or mp3.
        suffix = Path(path).suffix
        assert suffix in ['.wav', '.mp3'], f"Unsupported file format: {path}. It must be wav or mp3."
        if suffix == '.wav':
            return self._get_wave_info(path)
        elif suffix == '.mp3':
            return self._get_mp3_info(path)
        return 0, 0, 0

    def _get_wave_info(self, filepath: str) -> Tuple[int, int, int]:
        """ Gets info from a wave file.

        Returns:
            nchannels: Number of channels
            framerate: Number of frames per second. Same as the sampling rate.
            duration_in_ms: Total length of the audio in milliseconds

        Raises:
            FileNotFoundError: if the file is not found.
            wave.Error: if the file doesn't read properly.
        """
        
        f = sf.SoundFile(filepath)
        nframes = f.frames
        nchannels = f.channels
        framerate = f.samplerate
        if framerate == 0:
            raise InvalidFileError()
        
        duration_in_ms = int(nframes * 1000.0 / float(framerate))
        return nchannels, framerate, duration_in_ms


    def _get_mp3_info(self, filepath: str) -> Tuple[int, int, int]:
        """ Gets info from a mp3 file.

        Returns:
            nchannels: Number of channels
            framerate: Number of frames per second. Same as the sampling rate.
            duration_in_ms: Total length of the audio in milliseconds

        Raises:
            FileNotFoundError: if the file is not found.
        """
        # TODO parse the mp3 without pydub, which installs ffmpeg and causes lots of error.
        f = sf.SoundFile(filepath)
        nframes = f.frames
        nchannels = f.channels
        framerate = f.samplerate
        if framerate == 0:
            raise InvalidFileError()
        
        duration_in_ms = int(nframes * 1000.0 / float(framerate))
        return nchannels, framerate, duration_in_ms
        # return 0, 0, 0

class Audio:
    _path: str
    _name: str
    _remote_name: str
    _metadata: AudioMeta
    _upload_start_at: Optional[str] = None
    _upload_finish_at: Optional[str] = None
    _tags: Optional[List[str]] = None
    _group: Optional[str] = None
    _type: QuestionFileType = QuestionFileType.STIMULUS
    
    def __init__(
        self, 
        path: str,
        name: str, 
        remote_name: str,
        tags: Optional[List[str]],
        group: Optional[str],
        type: QuestionFileType
    ) -> None:
        self._path = path
        self._name = name
        self._remote_name = remote_name
        self._metadata = AudioMeta(path)
        self._tags = tags
        self._group = group
        self._type = type

    @property
    def path(self) -> str:
        return self._path

    @property
    def name(self) -> str:
        return self._name
    
    @property
    def remote_name(self) -> str:
        return self._remote_name
    
    @property
    def tags(self) -> Optional[List[str]]:
        return self._tags
    
    @property
    def group(self) -> Optional[str]:
        return self._group
    
    @property
    def metadata(self) -> AudioMeta:
        return self._metadata
    
    @property
    def type(self) -> QuestionFileType:
        return self._type

    def set_upload_at(self, start_at: str, finish_at: str) -> None:
        self._upload_start_at = start_at
        self._upload_finish_at = finish_at
    
    def to_dict(self) -> Dict[str, Any]:
        return {
            "name": self._name,
            "remote_name": self._remote_name, 
            "nchannels": self._metadata.nchannels,
            "framerate": self._metadata.framerate,
            "duration_in_ms": self._metadata.duration_in_ms,
            "upload_start_at": self._upload_start_at,
            "upload_finish_at": self._upload_finish_at,
            "tag": self._tags,
            "group": self._group
        }


import logging
from datetime import datetime, timedelta
from typing import Any, Dict, Optional

from podonos.common.constant import PODONOS_CONTACT_EMAIL
from podonos.common.enum import EvalType, Language

logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)


class EvalConfigDefault:
    TYPE = EvalType.NMOS
    LAN = Language.ENGLISH_AMERICAN
    NUM_EVAL = 3
    DUE_HOURS = 12
    AUTO_START = False
    GRANULARITY = 1.0


class EvalConfig:
    _eval_id: str 
    _eval_name: str
    _eval_expected_due: str  # Due string in ISO 8601.
    _eval_creation_timestamp: str  # Create a mission timestamp string. Use this as a prefix of uploaded filenames.
    _eval_description: Optional[str] = None
    _eval_type: EvalType = EvalConfigDefault.TYPE
    _eval_language: Language = EvalConfigDefault.LAN
    _eval_granularity: float = EvalConfigDefault.GRANULARITY
    _eval_num: int = 3
    _eval_expected_due_tzname: Optional[str] = None
    _eval_auto_start: bool = False

    def __init__(
        self,
        name: Optional[str] = None,
        desc: Optional[str] = None,
        type: str = EvalConfigDefault.TYPE.value,
        lan: str = EvalConfigDefault.LAN.value,
        granularity: float = EvalConfigDefault.GRANULARITY,
        num_eval: int = EvalConfigDefault.NUM_EVAL,
        due_hours: int = EvalConfigDefault.DUE_HOURS,  # TODO: allow floating point hours, e.g. 0.5.
        auto_start: bool = EvalConfigDefault.AUTO_START
    ) -> None:
        self._eval_name = self._set_eval_name(name)
        self._eval_description = desc
        self._eval_type = self._set_eval_type(type)
        self._eval_language = self._set_eval_language(lan)
        self._eval_num = self._set_eval_num(num_eval)
        self._eval_granularity = self._set_eval_granularity(granularity)
        self._eval_expected_due = self._set_eval_expected_due(due_hours)
        self._eval_expected_due_tzname = self._set_eval_expected_due_tzname()
        self._eval_creation_timestamp = self._set_eval_creation_timestamp()
        self._eval_id = self._eval_creation_timestamp
        self._eval_auto_start = self._set_eval_auto_start(auto_start)
        self.log_eval_config()

    def log_eval_config(self) -> None:
        log.debug(f'Name: {self._eval_name}')
        log.debug(f'Desc: {self._eval_description}')
        log.debug(f'Eval type: {self._eval_type}')
        log.debug(f'Language: {self._eval_language}')
        log.debug(f'num_eval: {self._eval_num}')
        log.debug(f'Expected due: {self._eval_expected_due} {self._eval_expected_due_tzname}')
        log.debug(f'Evaluation ID: {self._eval_id}')
        log.debug(f'Evaluation auto start: {self._eval_auto_start}')

    @property
    def eval_id(self) -> str:
        return self._eval_id
    
    @property
    def eval_type(self) -> EvalType:
        return self._eval_type
    
    @property
    def eval_creation_timestamp(self) -> str:
        return self._eval_creation_timestamp

    @property
    def eval_auto_start(self) -> bool:
        return self._eval_auto_start
    
    @eval_id.setter
    def eval_id(self, eval_id: str) -> None:
        self._eval_id = eval_id

    def _set_eval_name(self, eval_name: Optional[str]) -> str:
        if not eval_name:
            current = datetime.now()
            return f"{current.year}{current.month}{current.day}{current.hour}{current.minute}{current.second}"
        elif len(eval_name) > 1:
            return eval_name
        else:
            raise ValueError('"name" must be longer than 1.')

    def _set_eval_type(self, eval_type: str) -> EvalType:
        if eval_type not in [
            EvalType.NMOS.value, 
            EvalType.QMOS.value, 
            EvalType.SMOS.value, 
            EvalType.P808.value,
            EvalType.PREF.value,
        ]:
            raise ValueError(
                f'"type" must be one of {{NMOS, QMOS, SMOS, P808}}. \n' +
                f'Do you want other evaluation types? Let us know at {PODONOS_CONTACT_EMAIL}'
            )
        return EvalType(eval_type)

    def _set_eval_language(self, eval_language: str) -> Language:
        if eval_language not in [
            Language.ENGLISH_AMERICAN.value, Language.ENGLISH_AUSTRALIAN.value,
            Language.ENGLISH_BRITISH.value, Language.ENGLISH_CANADIAN.value,
            Language.KOREAN.value, Language.MANDARIN.value, 
            Language.SPANISH_SPAIN.value, Language.SPANISH_MEXICO.value,
            Language.FRENCH.value, Language.GERNAM.value,
            Language.JAPANESE.value, Language.ITALIAN.value,
            Language.POLISH.value, Language.AUDIO.value
        ]:
            raise ValueError(
                f'"lan" must be one of the supported language strings. ' +
                f'See https://docs.podonos.com/reference#create-evaluator \n' +
                f'Do you want us to support other languages? Let us know at {PODONOS_CONTACT_EMAIL}.'
            )
        return Language(eval_language)

    def _set_eval_num(self, num_eval: int) -> int:
        if num_eval < 1:
            raise ValueError(f'"num_eval" must be >= 1.')
        return num_eval

    # TODO: allow floating point hours, e.g. 0.5.
    def _set_eval_expected_due(self, due_hours: int) -> str:
        if due_hours < 12:
            raise ValueError('"due_hours" must be >=12.')

        due = datetime.now() + timedelta(hours=due_hours)
        return due.astimezone().isoformat(timespec='milliseconds')

    def _set_eval_expected_due_tzname(self) -> Optional[str]:
        return datetime.now().astimezone().tzname()

    def _set_eval_granularity(self, granularity: float) -> float:
        if granularity not in [0.5, 1.0]:
            raise ValueError(f'"granularity" must be one of 0.5 and 1.9')
        return granularity

    def _set_eval_creation_timestamp(self) -> str:
        return datetime.now().isoformat(timespec='milliseconds')

    def _set_eval_auto_start(self, eval_auto_start: bool) -> bool:
        return eval_auto_start

    def to_dict(self) -> Dict[str, Any]:
        return {
            'eval_id': self._eval_id,
            'eval_name': self._eval_name,
            'eval_description': self._eval_description,
            'eval_type': self._eval_type.value,
            'eval_language': self._eval_language.value,
            'eval_num': self._eval_num,
            'eval_expected_due': self._eval_expected_due,
            'eval_creation_timestamp': self._eval_creation_timestamp,
            'eval_auto_start': self._eval_auto_start
        }
    
    def to_create_request_dto(self) -> Dict[str, Any]:
        return {
            "title": self._eval_name,
            "internal_name": self._eval_name,
            "description": self._eval_description,
            "language": self._eval_language.value,
            "granularity": self._eval_granularity,
            "evaluation_type": self._eval_type.get_type()
        }



from typing import List

class File:
    _path: str
    _tags: List[str]
    
    def __init__(self, path: str, tags: List[str] = []) -> None:
        """
        Args:
            path: Path to the file to evaluate.
            tags: A list of string for file. Optional.
        """
        self._path = path
        self._tags = tags
    
    @property
    def path(self) -> str:
        return self._path
    
    @property
    def tags(self) -> List[str]:
        return self._tags


import datetime
import logging
import os
import requests
from abc import ABC, abstractmethod
from typing import Tuple, Dict, List, Optional

from podonos.common.constant import *
from podonos.common.enum import EvalType, QuestionFileType
from podonos.common.exception import HTTPError
from podonos.core.api import APIClient
from podonos.core.audio import Audio
from podonos.core.config import EvalConfig
from podonos.core.evaluation import Evaluation
from podonos.core.file import File

# For logging
logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

class Evaluator(ABC):
    """Evaluator for a single type of evaluation session."""
    _initialized: bool = False
    _api_client: APIClient
    _api_key: Optional[str] = None
    _eval_config: Optional[EvalConfig] = None
    _supported_evaluation_type: List[EvalType]

    # Contains the metadata for all the audio files for evaluation.
    _eval_audios: List[List[Audio]] = []
    _eval_audio_json = []

    def __init__(
        self, 
        api_client: APIClient,
        eval_config: Optional[EvalConfig] = None
    ):
        self._api_client = api_client
        self._api_key = api_client.api_key
        self._eval_config = eval_config
        self._initialized = True
        self._eval_audios = []
        self._eval_audio_json = []

    def _init_eval_variables(self):
        """Initializes the variables for one evaluation session."""
        self._initialized = False
        self._api_key = None
        self._eval_config = None
        self._eval_audios = []
        self._eval_audio_json = []
    
    @abstractmethod
    def add_file(
        self, 
        file: File
    ) -> None:
        pass
    
    @abstractmethod
    def add_file_pair(
        self, 
        target: File,
        ref: File
    ) -> None:
        pass
    
    @abstractmethod
    def add_file_set(
        self, 
        file0: File,
        file1: File
    ) -> None:
        pass
    
    def close(self) -> Dict[str, str]:
        """Closes the file uploading and evaluation session.
        This function holds until the file uploading finishes.

        Returns:
            JSON object containing the uploading status.

        Raises:
            ValueError: if this function is called before calling init().
        """
        if not self._initialized or self._eval_config is None:
            raise ValueError("No evaluation session is open.")
        
        # Upload files
        # TODO: add lazy & background upload
        if self._eval_config.eval_type not in self._supported_evaluation_type:
            raise ValueError("Not supported evaluation type")
        
        evaluation = self._create_evaluation()
        self._upload_files(evaluation_id=evaluation.id)

        # Create a json.
        session_json = self._eval_config.to_dict()
        session_json['files'] = self._eval_audio_json

        # Get the presigned URL for filename
        remote_object_name = os.path.join(self._eval_config.eval_creation_timestamp, 'session.json')
        presigned_url = self._get_presigned_url_for_put_method(evaluation.id, remote_object_name, 0, QuestionFileType.REF)
        try:
            response = self._api_client.put_json_presigned_url(url=presigned_url, data=session_json, headers={'Content-type': 'application/json'})
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP Error: {e}")
            raise HTTPError(f"Failed to upload session.json: {e}", status_code=e.response.status_code if e.response else None)
        
        if self._eval_config.eval_auto_start:
            print(f'{TerminalColor.OK}Upload finished. The evaluation will start immediately.{TerminalColor.ENDC}')
        else:
            print(f'{TerminalColor.OK}Upload finished. Please start the evaluation at {PODONOS_WORKSPACE}.{TerminalColor.ENDC}')

        # Initialize variables.
        self._init_eval_variables()
        return {'status': 'ok'}

    def _get_eval_config(self) -> EvalConfig:
        if not self._eval_config:
            raise ValueError("Evaluator is not initialized")
        return self._eval_config

    def _create_evaluation(self) -> Evaluation:
        """
        Create a new evaluation based on evaluation configuration

        Raises:
            HTTPError: If the value is invalid

        Returns:
            Evaluation: Get new evaluation information
        """
        
        eval_config = self._get_eval_config()
        try:
            response = self._api_client.post("evaluations", data=eval_config.to_create_request_dto())
            response.raise_for_status()
            evaluation = Evaluation.from_dict(response.json())
            eval_config.eval_id = evaluation.id
            return evaluation
        except Exception as e:
            raise HTTPError(f"Failed to create the evaluation: {e}")
    
    def _upload_files(self, evaluation_id: str) -> None:
        """Uploads the files for evaluation.
        This function holds until the file uploading finishes.

        Returns: None

        Raises:
            ValueError: if this function is called before calling init().
        """
        if not self._initialized or self._eval_config is None:
            raise ValueError("No evaluation session is open.")
        
        for audio_list in self._eval_audios:
            audio_json_list = []
            for audio in audio_list:
                # Get the presigned URL for filename
                upload_start_at, upload_finish_at = self._upload_one_file(
                    evaluation_id=evaluation_id, 
                    remote_object_name=audio.remote_name, 
                    path=audio.path,
                    duration_in_ms=audio.metadata.duration_in_ms,
                    tags=audio.tags if audio.tags else [],
                    type=audio.type,
                    group=audio.group
                )
                audio.set_upload_at(upload_start_at, upload_finish_at)
                audio_json_list.append(audio.to_dict())
            
            self._eval_audio_json.append(audio_json_list)

    def _upload_one_file(
        self, 
        evaluation_id: str, 
        remote_object_name: str, 
        path: str,
        duration_in_ms: int,
        type: QuestionFileType,
        tags: List[str] = [],
        group: Optional[str] = None,
    ) -> Tuple[str, str]:
        """
        Upload one file to server.

        Args:
            evaluation_id: New evaluation's id.
            remote_object_name: Path to the remote file name.
            path: Path to the local file.
            tags: Specific name list about file.
            type: Type used in QuestionFile.
            group: Group's name for combining with other files.
        Returns:
            upload_start_at: Upload start time in ISO 8601 string.
            upload_finish_at: Upload start time in ISO 8601 string.
        """
        # Get the presigned URL for files
        presigned_url = self._get_presigned_url_for_put_method(evaluation_id, remote_object_name, duration_in_ms, type, tags, group)

        # Timestamp in ISO 8601.
        upload_start_at = datetime.datetime.now().astimezone().isoformat(timespec='milliseconds')
        self._api_client.put_file_presigned_url(presigned_url, path)
        upload_finish_at = datetime.datetime.now().astimezone().isoformat(timespec='milliseconds')
        return upload_start_at, upload_finish_at
    
    def _get_presigned_url_for_put_method(
        self, 
        evaluation_id: str, 
        remote_object_name: str,
        duration_in_ms: int,
        type: QuestionFileType,
        tags: List[str] = [],
        group: Optional[str] = None,
    ) -> str:
        try:
            response = self._api_client.put(f"evaluations/{evaluation_id}/uploading-presigned-url", {
                "filename": remote_object_name,
                "duration": duration_in_ms,
                "tags": tags,
                "type": type,
                "group": group
            })
            response.raise_for_status()
            return response.text.replace('"', '')
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP Error: {e}")
            raise HTTPError(f"Failed to get presigned URL for {remote_object_name}: {e}", status_code=e.response.status_code if e.response else None)
    
    def _post_request_evaluation(self) -> None:        
        eval_config = self._get_eval_config()
        try:
            response = self._api_client.post("request-evaluation", {
                'eval_id': eval_config.eval_id,
                'eval_type': eval_config.eval_type.get_type()
            })
            response.raise_for_status()
        except requests.exceptions.HTTPError as e:
            log.error(f"HTTP Error: {e}")
            raise HTTPError(f"Failed to request evaluation: {e}", status_code=e.response.status_code if e.response else None)
    
    def _set_audio(self, path: str, tags: Optional[List[str]], group: Optional[str], type: QuestionFileType) -> Audio:
        valid_path = self._validate_path(path)
        name, remote_name = self._get_name_and_remote_name(valid_path)
                
        log.debug(f'remote_object_name: {remote_name}\n')
        return Audio(
            path=valid_path, name=name, remote_name=remote_name,
            tags=tags, group=group, type=type
        )
    
    def _validate_path(self, path: str) -> str:
        if not os.path.isfile(path):
            raise FileNotFoundError(f"File {path} doesn't exist")
            
        if not os.access(path, os.R_OK):
            raise FileNotFoundError(f"File {path} isn't readable")
        return path
    
    def _get_name_and_remote_name(self, valid_path: str) -> Tuple[str, str]:
        eval_config = self._get_eval_config()
        name = os.path.basename(valid_path)
        remote_name = os.path.join(eval_config.eval_creation_timestamp, name)
        return name, remote_name


import os
import podonos
import requests
import logging
import importlib.metadata

from requests import Response
from typing import Dict, Any, Optional
from packaging.version import Version

from podonos.common.constant import *
from podonos.common.exception import HTTPError


logging.basicConfig(level=logging.INFO)
log = logging.getLogger(__name__)

class APIVersion:
    _minimum: Version
    _recommended: Version
    _latest: Version
    
    def __init__(self, minimum: str, recommended: str, latest: str):
        self._minimum = Version(minimum)
        self._recommended = Version(recommended)
        self._latest = Version(latest)
    
    @property
    def minimum(self) -> Version:
        return self._minimum
    
    @property
    def recommended(self) -> Version:
        return self._recommended
    
    @property
    def latest(self) -> Version:
        return self._latest

class APIClient:
    _api_key: str
    _api_url: str
    _headers: Dict[str, str] = {}
    
    def __init__(self, api_key: str, api_url: str):
        self._api_key = api_key
        self._api_url = api_url
        self._headers = {
            "X-API-KEY": self._api_key
        }
    
    @property
    def api_key(self) -> str:
        return self._api_key
    
    @property
    def api_url(self) -> str:
        return self._api_url
    
    def initialize(self) -> bool:
        self._check_minimum_version()
        
        response = self.get("customers/verify/api-key")
        if response.text != "true":
            raise ValueError(TerminalColor.FAIL + f"Invalid API key: {self._api_key}"  + TerminalColor.ENDC)
        return True
    
    def add_headers(self, key: str, value: str) -> None:
        self._headers[key] = value
    
    def get(self, endpoint: str, params: Optional[Dict[str, str]] = None, headers: Optional[Dict[str, str]] = None) -> Response:
        request_header = self._headers if headers is None else headers
        response = requests.get(f"{self._api_url}/{endpoint}", headers=request_header, params=params)
        return response

    def post(self, endpoint: str, data: Dict[str, Any], headers: Optional[Dict[str, str]] = None) -> Response:
        request_header = self._headers if headers is None else headers
        response = requests.post(f"{self._api_url}/{endpoint}", headers=request_header, json=data)
        return response
    
    def put(self, endpoint: str, data: Dict[str, Any], headers: Optional[Dict[str, str]] = None) -> Response:
        request_header = self._headers if headers is None else headers
        response = requests.put(f"{self._api_url}/{endpoint}", headers=request_header, json=data)
        return response
    
    def put_file_presigned_url(self, url: str, path: str) -> Response:
        try:
            response = requests.put(url, data=open(path, "rb"), headers={
                "Content-Type": self._set_content_type_by_filename(path)
            })
            return response
        except requests.exceptions.RequestException as e:
            log.error(f"HTTP Error: {e}")
            raise HTTPError(f"Failed to Upload File {path}: {e}",
                            status_code=e.response.status_code if e.response else None)
    
    def put_json_presigned_url(self, url: str, data: Dict[str, Any], headers: Optional[Dict[str, str]] = None) -> Response:
        try:
            response = requests.put(url, json=data, headers=headers)
            return response
        except requests.exceptions.RequestException as e:
            log.error(f"HTTP Error: {e}")
            raise HTTPError(f"Failed to Upload JSON {data}: {e}",
                            status_code=e.response.status_code if e.response else None)
        
    def _set_content_type_by_filename(self, path: str) -> str:
        _, ext = os.path.splitext(path)
        if ext == '.wav':
            return "audio/wav"
        elif ext == '.mp3':
            return "audio/mpeg"
        elif ext == '.json':
            return "application/json"
        return "application/octet-stream"
    
    def _check_minimum_version(self) -> bool:
        response = self.get("version/sdk")
        api_version = APIVersion(**response.json())
        
        current_version = self._get_podonos_version()
        log.debug(f'current package version: {current_version}')
        
        if Version(current_version) >= api_version.recommended:
            return True
        
        if Version(current_version) >= api_version.minimum:
            print(
                "The current podonos package version is {current_version} "
                "while a newer version {api_version.latest} is available\n"
                "Please upgrade by 'pip install podonos --upgrade'"
            )
            return True
        
        # This version is lower than the minimum required version. Cannot proceed.
        print(
            TerminalColor.FAIL +
            f"The current podonos package version is {current_version} "
            f"while the minimum supported version is {api_version.minimum}" + TerminalColor.ENDC + "\n" +
            TerminalColor.BOLD + "Please upgrade" + TerminalColor.ENDC + f" by 'pip install podonos --upgrade'"
        )
        raise ValueError(f"Minimum supported version is {api_version.minimum}")

    def _get_podonos_version(self):
        try:
            # Try to get the version using importlib.metadata
            return importlib.metadata.version("podonos")
        except importlib.metadata.PackageNotFoundError:
            # Fallback to __version__ from podonos package if importlib fails
            return podonos.__version__